@@ -9,6 +9,11 @@ use tokio::{
 };
 use uuid::Uuid;
 
+pub enum StoppingResult {
+    Recover,
+    Stop,
+}
+
 #[async_trait]
 pub trait Actor: Send + Sync + Sized {
     type Msg: 'static + Send + Sync;
@@ -17,6 +22,13 @@ pub trait Actor: Send + Sync + Sized {
 
     async fn init(&mut self, _ctx: &mut Context<Self>) {}
 
+    /// Called after ctx.stop() is called.
+    ///
+    /// Can be used to restart try and recover the actor and restart the run loop.
+    async fn stopping(&mut self, _ctx: &mut Context<Self>) -> StoppingResult {
+        StoppingResult::Stop
+    }
+
     async fn stopped(self, _ctx: &mut Context<Self>) {}
 }
 
@@ -111,8 +123,17 @@ impl Agency {
         self.spawner.spawn(async move {
             actor.init(&mut ctx).await;
 
-            while !ctx.stopped {
-                actor.run(&mut ctx).await;
+            loop {
+                while !ctx.stopped {
+                    actor.run(&mut ctx).await;
+                }
+
+                match actor.stopping(&mut ctx).await {
+                    StoppingResult::Recover => ctx.stopped = false,
+                    StoppingResult::Stop => {
+                        break;
+                    }
+                }
             }
 
             actor.stopped(&mut ctx).await;
@@ -130,8 +151,17 @@ impl Agency {
             if let Some(mut actor) = A::setup(&mut ctx, args).await {
                 actor.init(&mut ctx).await;
 
-                while !ctx.stopped {
-                    actor.run(&mut ctx).await;
+                loop {
+                    while !ctx.stopped {
+                        actor.run(&mut ctx).await;
+                    }
+
+                    match actor.stopping(&mut ctx).await {
+                        StoppingResult::Recover => ctx.stopped = false,
+                        StoppingResult::Stop => {
+                            break;
+                        }
+                    }
                 }
 
                 actor.stopped(&mut ctx).await;
